# AddRedirector - A Plugin for Movable Type 4.1
#
# Release 0.11
# February 22, 2008
#
# Author: nlog(n)
# Email: n [at] nlogn [dot] ath [dot] cx
#
package MT::Plugin::AddRedirector;

use strict;
use base qw(MT::Plugin);
use vars qw($VERSION);
$VERSION = '0.11';

my $plugin = new MT::Plugin::AddRedirector({
    name => 'AddRedirector',
    version => $VERSION,
    author_name => 'nlog(n)',
    author_link => 'http://nlogn.ath.cx/',
    plugin_link => 'http://nlogn.ath.cx/plugins/addredirector/',
    description => '<MT_TRANS phrase="Adds redirector to hyperlinks which link to the outside of the blog.">',
    settings => new MT::PluginSettings([
        ['redirector', { Default => '' }],
        ['exclude', { Default => '' }],
    ]),
    blog_config_template => \&blog_config_template,
    l10n_class => 'AddRedirector::L10N',
});
MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tags => {
            block => {
                'AddRedirector' => \&AddRedirector,
            },
            modifier => {
                'urlredirect' => \&RedirectFilter,
            },
        }
   });
}

sub blog_config_template {
  my ($plugin, $param, undef) = @_;
  return <<'TMPL';
<mt:App:Setting
    id="redirector"
    label="<__trans phrase="Redirector">">
    <div class="textarea-wrapper">
        <input name="redirector" id="redirector" class="full-width" size="30" value="<TMPL_VAR NAME=REDIRECTOR ESCAPE=HTML>" />
    </div>
</mt:App:Setting>
<mt:App:Setting
    id="exclude"
    label="<__trans phrase="Exclude">">
    <div class="textarea-wrapper">
        <input name="exclude" id="exclude" class="full-width" size="30" value="<TMPL_VAR NAME=EXCLUDE ESCAPE=HTML>" />
    </div>
</mt:App:Setting>
TMPL
}

sub AddRedirector {
  my ($ctx, $args) = @_;

  my $tokens = $ctx->stash('tokens');
  my $builder = $ctx->stash('builder');

  defined(my $out = $builder->build($ctx, $tokens))
      or return $ctx->error($builder->errstr);

  my $config = $plugin->get_config_hash('blog:' . $ctx->stash('blog')->id);
  my $redirector = $config->{redirector};
  my $exclude = $config->{exclude} || $ctx->stash('blog')->site_url;

  $out =~ s/(<a .*?>)/addredirector($1, $redirector, $exclude)/eigs;
  $out;
}

sub RedirectFilter {
  my ($out, $args, $ctx) = @_;

  my $config = $plugin->get_config_hash('blog:' . $ctx->stash('blog')->id);
  my $redirector = $config->{redirector};
  my $exclude = $config->{exclude} || $ctx->stash('blog')->site_url;

  $out =~ s/(<a .*?>)/addredirector($1, $redirector, $exclude)/eigs;
  $out;
}

sub addredirector {
  my ($a, $redirector, $exclude) = @_;

  my @excludes = split /\|/, $exclude;
  if ($a =~ m/href="(.*?)"/i
      && substr($1, 0, length($redirector)) ne $redirector
      && !(() = grep {$1 =~ /$_/} @excludes)) {
    $a =~ s/href="(.*?)"/href="$redirector$1"/i;
  }
  $a;
}

1;
